@@ -2,7 +2,7 @@
 
 #  Simon Drabble	03/22/02
 #  sdrabble@cpan.org
-#  $Id: snagmail.pl,v 1.10 2002/10/28 19:18:19 simon Exp $
+#  $Id: snagmail.pl,v 1.13 2003/01/18 03:53:07 simon Exp $
 #
 
 
@@ -42,9 +42,36 @@ if (lc $ARGV[2] eq 'all') {
 	$msg_list = \@fetchnums;
 }
 
-my @messages = $yahoo->get_mail_messages('Inbox', $msg_list);
+my $flags = 0;
+my $move_to = '';
+
+# If you wish to delete messages on server after downloading, uncomment this:
+$flags |= DELETE_ON_READ;
+
+# If you wish to move messages after downloading, uncomment the next 2 lines:
+# Note that MOVE_ON_READ has precedence over DELETE_ON_READ.
+#$flags |= MOVE_ON_READ;
+#$move_to = "INSERT YOUR FOLDER NAME HERE"; 
+
+
+if ($flags & MOVE_ON_READ && $flags & DELETE_ON_READ) {
+	$flags ^= DELETE_ON_READ;
+}
+
+
+my @messages = $yahoo->get_mail_messages('Inbox', $msg_list, $flags, $move_to);
 print "Message Headers in Inbox: ", 0+@messages, "\n";
 print "Messages will be delivered to ./${name}_Inbox.\n";
+if ($flags & DELETE_ON_READ && !($flags & MOVE_ON_READ)) {
+	print "Messages will be deleted from server after download.\n";
+}
+if ($move_to) {
+	if ($flags & MOVE_ON_READ) {
+		print "Messages will be moved to $move_to after download.\n";
+	}
+}
+
+
 open INBOX, ">${name}_Inbox";
 for (@messages) { print INBOX $_->as_mbox_string }
 close INBOX;
@@ -56,7 +83,7 @@ print "\n";
 sub usage
 {
 	print qq{
-$0 <yahoo username> <yahoo password>  [<number of messages to download>]
+$0 <username> <password>  [<list of messages to retrieve>]
 };	
 }
 
